// MatchingVarargsMethod.aj
//
// Informatics 102 Spring 2012
// Code Example
//
// This aspect demonstrates matching the varargs method String.format(),
// including how to hoist its arguments -- including the variable-length
// part of the argument list -- out of join points and into advice.

public aspect MatchingVarargsMethod
{
	// This is how you match calls to varargs methods.  Note that it's not
	// that different from matching calls to other methods, except the use
	// of "..." in the last position of the list of argument types.
	//
	// One interesting caveat, though, is that pointcut parameters don't
	// support a variable number of arguments like methods do, so AspectJ
	// allows you to hoist the variable-length part of the argument list
	// into an array instead.  Note, then, how the Object... arguments
	// in String.format() are turned into a pointcut parameter of type
	// Object[].
	pointcut callToStringFormatMethod(String s, Object[] objs):
		call(String String.format(String, Object...)) && args(s, objs);


	// Advice is also unable to accept a variable number of arguments,
	// so you would also want to use the array syntax here to match
	// the array coming out of the pointcut parameter.  This example
	// advice just prints out the format string and the remaining
	// arguments to System.out.
	before(String s, Object[] objs):
		callToStringFormatMethod(s, objs)
	{
		System.out.print("String.format(" + s);
		
		for (Object obj : objs)
		{
			System.out.print(", " + obj);
		}
		
		System.out.println(")");
	}
}
